@@ -85,10 +85,21 @@ use Jifty::Web::Menu ();
 use Jifty::Subs ();
 use Jifty::Subs::Render ();
 
+use Jifty::CAS ();
+
 use Jifty::Module::Pluggable;
 #Jifty::Module::Pluggable->import(search_path => ['Jifty::Web::Form::Field'], require     => 1, except      => qr/\.#/);
 #__PACKAGE__->plugins;
 
+# Set up to load commands defined in Jifty/Plugin/*/Command/*.pm
+# we do the actual load in Jifty::Script
+Jifty::Module::Pluggable->import(
+    search_path => ['Jifty::Plugin'],
+    file_regex  => qr{/Command/[^/]+},
+    require     => 1,
+    sub_name    => "plugin_commands"
+);
+
 =head1 SEE ALSO
 
 L<Jifty>
