@@ -48,7 +48,8 @@ use Jifty::Plugin::Authentication::Password::Record schema {
 
 column auth_token =>
   render_as 'unrendered',
-  type is 'varchar',
+  type is 'varchar(255)',
+  max_length is 255,
   default is '',
   label is _('Authentication token');
     
@@ -57,7 +58,8 @@ column auth_token =>
 column password =>
   is unreadable,
   label is _('Password'),
-  type is 'varchar',
+  type is 'varchar(64)',
+  max_length is 64,
   hints is _('Your password should be at least six characters'),
   render_as 'password',
   filters are 'Jifty::DBI::Filter::SaltHash';
