@@ -4,25 +4,25 @@ use strict;
 
 use Jifty::Dispatcher -base;
 
-# http://your.app/leaks -- display full leak report
-on 'leaks' => run {
+# http://your.app/__jifty/admin/leaks -- display full leak report
+on '/__jifty/admin/leaks' => run {
     set 'skip_zero' => 1;
-    show "leaks/all";
+    show "/__jifty/admin/leaks/all";
 };
 
-# http://your.app/leaks/all -- full leak report with non-leaked requests
-on 'leaks/all' => run {
+# http://your.app/__jifty/admin/leaks/all -- leak report with 0-leak requests
+on '/__jifty/admin/leaks/all' => run {
     set 'skip_zero' => 0;
-    show "leaks/all";
+    show "/__jifty/admin/leaks/all";
 };
 
-# http://your.app/leaks/xxx -- display leak report for request ID xxx
-on 'leaks/#' => run {
+# http://your.app/__jifty/admin/leaks/xxx -- display leak report for request ID
+on '/__jifty/admin/leaks/#' => run {
     abort(404) if $1 < 1;
     my $leak = $Jifty::Plugin::LeakTracker::requests[$1 - 1]
         or abort(404);
     set leak => $leak;
-    show "leaks/one";
+    show "/__jifty/admin/leaks/one";
 };
 
 =head1 SEE ALSO
