@@ -418,8 +418,6 @@ private template edit_item_controls => sub {
     my $delete = $record->as_delete_action(
         moniker => 'delete-' . Jifty->web->serial,
     );
-    Jifty->web->form->register_action($delete);
-
         div {
             { class is 'crud editlink' };
             hyperlink(
@@ -438,13 +436,13 @@ private template edit_item_controls => sub {
                     args         => { object_type => $object_type, id => $id }
                 },
                 as_button => 1,
-                class => 'cancel'
+                class     => 'cancel'
             );
             if ( $record->current_user_can('delete') ) {
                 $delete->button(
                     label   => _('Delete'),
                     onclick => {
-                        submit => $delete,
+                        submit  => $delete,
                         confirm => _('Really delete?'),
                         refresh => Jifty->web->current_region->parent,
                     },
@@ -464,7 +462,7 @@ The list template provides an interactive list for showing a list of records in
 template 'list' => sub {
     my $self = shift;
 
-    my ( $page ) = get(qw(page ));
+    my ( $page ) = get('page');
     my $item_path = get('item_path') || $self->fragment_for("view");
     my $collection =  $self->_current_collection();
     div { {class is 'crud-'.$self->object_type}; 
@@ -491,15 +489,15 @@ sub per_page { 25 }
 # unlimited collection if there is no current search.
 sub _current_collection {
     my $self = shift; 
-    my ( $page, $search_collection ) = get(qw(page  search_collection));
+    my ( $page ) = get('page');
     my $collection_class = $self->record_class->collection_class;
-    my $search = $search_collection || ( Jifty->web->response->result('search') ? Jifty->web->response->result('search')->content('search') : undef );
+    my $search = ( Jifty->web->response->result('search') ? Jifty->web->response->result('search')->content('search') : undef );
     my $collection;
     if ( $search ) {
         $collection = $search;
     } else {
         $collection = $collection_class->new();
-        $collection->unlimit();
+        $collection->find_all_rows();
     }
 
     $collection->set_page_info( current_page => $page, per_page => $self->per_page );
@@ -583,6 +581,8 @@ private template 'list_items' => sub {
     my $item_path   = shift;
     my $callback    = shift;
     my $object_type = $self->object_type;
+    $collection->_do_search(); # we're going to need the results. 
+    # XXX TODO, should use a real API to force the search
     if ( $collection->count == 0 ) {
         show('./no_items_found');
     }
@@ -700,7 +700,7 @@ The new_item template provides a form for creating new model records. See L<Jift
 
 template 'new_item' => sub {
     my $self = shift;
-    my ( $object_type, $id ) = ( $self->object_type, get('id') );
+    my ( $object_type ) = ( $self->object_type );
 
     my $record_class = $self->record_class;
     my $create = $record_class->as_create_action;
