@@ -39,17 +39,14 @@ __PACKAGE__->javascript_libs([qw(
     jsan/DOM/Events.js
     json.js
     prototype.js
-    cssquery/cssQuery.js
-    cssquery/cssQuery-level2.js
-    cssquery/cssQuery-level3.js
-    cssquery/cssQuery-standard.js
+    jquery-1.2.1.js
+    jquery_noconflict.js
     behaviour.js
     scriptaculous/builder.js
     scriptaculous/effects.js
     scriptaculous/controls.js
     formatDate.js
     template_declare.js
-    loc.js
     jifty.js
     jifty_utils.js
     jifty_subs.js
@@ -73,6 +70,7 @@ __PACKAGE__->javascript_libs([qw(
     app.js
     app_behaviour.js
     css_browser_selector.js
+    cssQuery-jquery.js
 )]);
 
 use Class::Trigger;
@@ -248,21 +246,22 @@ sub current_user {
         $self->_current_user( $currentuser_obj || undef );
     }
 
+    my $object;
+
     if ( defined $self->temporary_current_user ) {
         return $self->temporary_current_user;
     } elsif ( defined $self->_current_user ) {
         return $self->_current_user;
-
     } elsif ( my $id = $self->session->get('user_id') ) {
-        my $object = Jifty->app_class("CurrentUser")->new( id => $id );
-        $self->_current_user($object);
-        return $object;
+         $object = Jifty->app_class("CurrentUser")->new( id => $id );
+    } elsif ( Jifty->config->framework('AdminMode')) {
+         $object = Jifty->app_class("CurrentUser")->superuser;
     } else {
-        my $object = Jifty->app_class("CurrentUser")->new;
-        $object->is_superuser(1) if Jifty->config->framework('AdminMode');
-        $self->_current_user($object);
-        return ($object);
+         $object = Jifty->app_class("CurrentUser")->new;
     }
+    
+    $self->_current_user($object);
+    return $object;
 }
 
 =head3 temporary_current_user [USER]
@@ -361,6 +360,7 @@ sub _validate_request_actions {
                         . $request_action->class
                         . "'" );
                 Carp::cluck;
+                $self->log->error("NOTICE! A cross-site scriptng security fix has been installed so that actions are now by default DENIED during GET requests. You must specifically whitelist safe actions using this in your dispatcher: before '*' => run { Jifty->api->allow('SafeAction') }; - We apologize for the inconvenience.");
                 push @denied_actions, $request_action;
                 next;
             }
@@ -386,57 +386,57 @@ sub _validate_request_actions {
 }
 
 sub _process_valid_actions {
-    my  $self = shift;
+    my $self          = shift;
     my $valid_actions = shift;
-        for my $request_action (@$valid_actions) {
+    for my $request_action (@$valid_actions) {
 
-            # Pull the action out of the request (again, since
-            # mappings may have affected parameters).  This
-            # returns the cached version unless the request has
-            # been changed by argument mapping from previous
-            # actions (Jifty::Request::Mapper)
-            my $action = $self->new_action_from_request($request_action);
-            next unless $action;
-            if ( $request_action->modified ) {
+        # Pull the action out of the request (again, since
+        # mappings may have affected parameters).  This
+        # returns the cached version unless the request has
+        # been changed by argument mapping from previous
+        # actions (Jifty::Request::Mapper)
+        my $action = $self->new_action_from_request($request_action);
+        next unless $action;
+        if ( $request_action->modified ) {
 
-                # If the request's action was changed, re-validate
-                $action->result( Jifty::Result->new );
-                $action->result->action_class( ref $action );
-                $self->response->result(
-                    $action->moniker => $action->result );
-                $self->log->debug( "Re-validating action "
-                        . ref($action) . " "
-                        . $action->moniker );
-                next unless $action->validate;
-            }
-
-            $self->log->debug(
-                "Running action " . ref($action) . " " . $action->moniker );
-            eval { $action->run; };
-            $request_action->has_run(1);
-
-            if ( my $err = $@ ) {
-
-                # Poor man's exception propagation; we need to get
-                # "LAST RULE" and "ABORT" exceptions back up to the
-                # dispatcher.  This is specifically for redirects from
-                # actions
-                die $err if ( $err =~ /^(LAST RULE|ABORT)/ );
-                $self->log->fatal($err);
-                $action->result->error(
-                    Jifty->config->framework("DevelMode")
-                    ? $err
-                    : _("There was an error completing the request.  Please try again later."
-                    )
-                );
-            }
-
-            # Fill in the request with any results that that action
-            # may have yielded.
-            $self->request->do_mapping;
+            # If the request's action was changed, re-validate
+            $action->result( Jifty::Result->new );
+            $action->result->action_class( ref $action );
+            $self->response->result( $action->moniker => $action->result );
+            $self->log->debug( "Re-validating action "
+                    . ref($action) . " "
+                    . $action->moniker );
+            next unless $action->validate;
         }
 
+        $self->log->debug(
+            "Running action " . ref($action) . " " . $action->moniker );
+        eval { $action->run; };
+        $request_action->has_run(1);
+
+        if ( my $err = $@ ) {
+
+            # Poor man's exception propagation; we need to get
+            # "LAST RULE" and "ABORT" exceptions back up to the
+            # dispatcher.  This is specifically for redirects from
+            # actions
+            die $err if ( $err =~ /^(LAST RULE|ABORT)/ );
+            $self->log->fatal($err);
+            $action->result->error(
+                Jifty->config->framework("DevelMode")
+                ? $err
+                : _("There was an error completing the request.  Please try again later."
+                )
+            );
+        }
+
+        # Fill in the request with any results that that action
+        # may have yielded.
+        $self->request->do_mapping;
     }
+
+}
+
 =head3 request [VALUE]
 
 Gets or sets the current L<Jifty::Request> object.
@@ -760,7 +760,7 @@ sub _redirect {
         local $self->{navigation} = undef;
         local $self->{page_navigation} = undef;
         $self->replace_current_region($page);
-        Jifty::Dispatcher::_abort;
+        Jifty::Dispatcher::_abort();
         return;
     }
 
@@ -792,7 +792,7 @@ sub _redirect {
 
     # Mason abort, or dispatcher abort out of here
     $self->mason->abort if $self->mason;
-    Jifty::Dispatcher::_abort;
+    Jifty::Dispatcher::_abort();
 }
 
 =head3 caller
@@ -1172,6 +1172,25 @@ sub add_javascript {
     Jifty->web->javascript_libs([ @{ Jifty->web->javascript_libs }, @_ ]);
 }
 
+=head3 remove_javascript FILE1, FILE2, ...
+
+Removes the given files from C<< Jifty->web->javascript_libs >>.
+
+This is intended for plugins or applications that provide another version of
+the functionality given in our default JS libraries. For example, the CSSQuery
+plugin will get rid of the cssQuery-jQuery.js back-compat script.
+
+=cut
+
+sub remove_javascript {
+    my $self = shift;
+    my %remove = map { $_ => 1 } @_;
+
+    Jifty->web->javascript_libs([
+        grep { !$remove{$_} } @{ Jifty->web->javascript_libs }
+    ]);
+}
+
 =head3 add_external_javascript URL1, URL2, ...
 
 Pushes urls onto C<Jifty->web->external_javascript_libs>
